#include <math.h>
#ifndef MAM_DSP_H_INCLUDED
#define MAM_DSP_H_INCLUDED

#define PI 3.1415926535897932384626433832795

int i,j;

///////////////////////////////
//                           //
//      1-POLE FILTERS       //
//                           //
///////////////////////////////

void LS_Filter(signed short* inWave, signed short* outWave, int len, double q)
// q - filter factor = 0.0 .. 1.0
// q = 0.0 - not filtered signal
// q = 1.0 - silence (filter all)
{
 double *in, *out, *temp;
 in = new double[len];
 out = new double[len];

 q = 1 - (1-q)*(1-q);
 
 for (i=0; i<len; i++)
 {
  in[i] = inWave[i];
 }

 for (j=0; j<8; j++)
 {
  out[0] = (1-q)*in[0];

  for (i=1; i<len; i++)
  { 
   out[i] = q*out[i-1] + (1-q)*in[i];
  }

  if(q)
  for (i=0; i<len; i++)
  {
   out[i] = out[i] - (1.0-q)*in[i]/(1.0+q);
   out[i] *= (1.0 + q) / (2.0 * q);
  }
 
  temp = in;
  in = out;
  out = temp;
 }

 for (i=0; i<len; i++)
 {
  if (in[i] < -32767) in[i] = -32767; 
  if (in[i] > 32767)  in[i] =  32767;
  outWave[i] = in[i];
 }

 delete [] in;
 delete [] out;
}

void HS_Filter(signed short* inWave, signed short* outWave, int len, double q)
// q - filter factor = 0.0 .. 1.0
// q = 0.0 - not filtered signal
// q = 1.0 - silence (filter all)
{
 double out, prev;

 q = 1 - (1-q)*(1-q);

 outWave[0] = prev = (1.0-q)*inWave[0];

 for (i=1; i<len; i++)
 { 
  out = (-q)*prev + (1-q)*inWave[i];
  if (out < -32767) out = -32767; 
  if (out > 32767)  out =  32767;
  outWave[i] = out;
  prev = out;
 }
}

///////////////////////////////
//                           //
//      2-POLE FILTERS       //
//                           //
///////////////////////////////

void Resonator(signed short* inWave, signed short* outWave, int len, double freq, double q, double decay)
// freq - frequency of resonance = 0 .. 22050
// q - resonance factor = 0.0 .. 1.0
// decay - decay of frequency = 0.0 .. 1.0
{
 double out, prev, pprev;
 double freq_real;

 outWave[0] = pprev = inWave[0];
 outWave[1] = prev  = 2*q*cos(PI*freq/22050)*inWave[0] + inWave[1];

 for (i=2; i<len; i++)
 { 
  freq_real = freq*(decay*(1.0 - double(i)/double(len)) + 1.0 - decay);
  out = 2*q*cos(PI*freq_real/22050)*prev - q*q*pprev + inWave[i];
  if (out < -32767) out = -32767; 
  if (out > 32767)  out =  32767;
  outWave[i] = out;
  prev = out;
  pprev = prev;
 }
}

///////////////////////////////
//                           //
//    NON-LINEAR PROCESSORS  //
//                           //
///////////////////////////////

void Slew_Compressor(signed short* inWave, signed short* outWave, int len, double c, double lim)
// c - compress factor = 0.0 .. 1.0
// lim - limitation of slew = 0.0 .. 1.0
{
 double prev, temp, out;
 double diff;

 lim = lim*32768;
 prev = inWave[0];
 outWave[0] = prev;

 for (i=1; i<len; i++)
 { 
  diff = inWave[i] - prev;
  if (diff > lim) diff = lim;
  if (diff < -lim) diff = -lim;
  out = prev + (1.0-c)*diff + c*diff*diff*diff/(32768*32768);
  if (out < -32767) out = -32767; 
  if (out > 32767)  out =  32767;
  outWave[i] = out;
  prev = out;
 }
}

void Distorter(signed short* inWave, signed short* outWave, int len, double c, double h, double cx)
// c - distort factor (center point) = 0.0 .. 1.0
// h - distort factor (high point)= 0.0 .. 1.0
// cx - position of center point = 0.0 .. 1.0
{
 double out;
 double t;
 
 if (cx<=0) cx = 0.001;

 for (i=0; i<len; i++)
 {
  t = abs(inWave[i]);
  t /= 32768;

  if (t < cx) t = (1.0/cx) * t * c;
  else t = c + (1.0/(1.0-cx))*(t - cx)*(h-c);

  if (inWave[i]<0) t *= -1;
  out = t*32768;
  if (out < -32767) out = -32767; 
  if (out > 32767)  out =  32767;
  outWave[i] = out;
 }
}

///////////////////////////////
//                           //
//    FM SYNTHESYS           //
//                           //
///////////////////////////////

void FM_Operator(signed short* inWave, signed short* outWave, int len, double freq, double amp, double fm)
// freq - operator frequency = 0 .. 22050
// amp - amplitude = 0.0 .. 1.0
// fm - modulation amount = 0 .. 22050
{
 double out;
 double t = 0;

 for (i=0; i<len; i++)
 {
  t += freq + fm*double(inWave[i])/32767.0;
  out = 32768.0*amp*sin(2.0*PI*t/44100.0);
  if (out < -32767) out = -32767; 
  if (out > 32767)  out =  32767;
  outWave[i] = out;
 }
}

///////////////////////////////
//                           //
//    SIMPLE PROCESSORS      //
//                           //
///////////////////////////////

void Multiply(signed short* inStage1, signed short* inStage2, signed short* outWave, int len)
// production of two signals
{
 for (i=0; i<len; i++)
 {
  outWave[i] = double(inStage1[i])*inStage2[i]/32767.0;
 }
}

///////////////////////////////
//                           //
//      FIR FILTERS          //
//                           //
///////////////////////////////

void HP_Filter(signed short* inWave, signed short* outWave, int len, double freq)
// freq = cutoff frequency = 0..22050
{
 double *in, *out, *temp;
 in = new double[len];
 out = new double[len];
 temp = new double[257];

 for (i=0; i<len; i++)
 {
  in[i] = inWave[i];
 }

 for (i=0; i<257; i++)
 {
  if(i!=129)
  temp[i] = - (1.0/(PI*(i-129))) * sin(PI*(i-129)*freq/22050);
 }
 temp[129] = 1.0 - freq/22050;

 for (i=0; i<len; i++)
 {
  out[i] = 0;
  for (j=-128; j<129; j++)
   if ((i+j >= 0)&&(i+j < len)) out[i] += in[i+j]*temp[j+128];
 }

 for (i=0; i<len; i++)
 {
  if (out[i] < -32767) out[i] = -32767; 
  if (out[i] > 32767)  out[i] =  32767;
  outWave[i] = out[i];
 }

 delete [] in;
 delete [] out;
 delete [] temp;
}



void LP_Filter(signed short* inWave, signed short* outWave, int len, double cutoff, double resonance)
// cutoff - 0 ... 127
// resonance - 0 ... 127
// filter posted on musicdsp.org by madbrain@videotron.ca

{
	double *in, *out, v0, v1;
	in = new double[len];
	out = new double[len];

	for (int i=0; i<len; i++)
	{
		in[i] = inWave[i];
	}

	double c = pow(0.5, (128-cutoff)   / 16.0);
	double r = pow(0.5, (resonance+24) / 16.0);

	for (i=0; i<len; i++)
	{
		v0 =  (1-r*c)*v0  -  (c)*v1  + (c)*in[i];
		v1 =  (1-r*c)*v1  +  (c)*v0;
		out[i] = v1;
		if (out[i] < -32767) out[i] = -32767; 
		if (out[i] > 32767)  out[i] =  32767;
		outWave[i] = out[i];
	}

	delete [] in;
	delete [] out;
}


void LP_Filter2(signed short* inWave, signed short* outWave, int len, double cutoff, double resonance,double cutoffSpd,double cutoffAmp,double rcSpd,double rcAmp,double Fz)
// cutoff - 0 ... 127
// resonance - 0 ... 127
// filter posted on musicdsp.org by madbrain@videotron.ca
{
	double *in, *out, v0, v1;
	in = new double[len];
	out = new double[len];

	for (int i=0; i<len; i++)
	{
		in[i] = inWave[i];
	}

	double c;
	double r;
	double rs,cs,F;
	F=((double)Fz)/255.0;

	for (i=0; i<len; i++)
	{
		rs=(rcAmp*sin(PI*rcSpd/32.0*((double)i/(double)len)+F))/2.0;
		cs=(cutoffAmp*sin(PI*cutoffSpd/32.0*((double)i/(double)len)+F))/2.0;

		c = pow(0.5, (128-cutoff+cs)   / 16.0);
		r = pow(0.5, (resonance+rs+24) / 16.0);

		v0 =  (1-r*c)*v0  -  (c)*v1  + (c)*in[i];
		v1 =  (1-r*c)*v1  +  (c)*v0;
		out[i] = v1;
		if (out[i] < -32767) out[i] = -32767; 
		if (out[i] > 32767)  out[i] =  32767;
		outWave[i] = out[i];
	}

	delete [] in;
	delete [] out;
}



void Normalize(signed short* inWave, signed short* outWave, int len)
// waveform normalizing
{
	double max;
	max = 0;

	for (i=0; i<len; i++)
	{
		if (inWave[i]>max) max = inWave[i];
		if (inWave[i]<-max) max = -inWave[i];
	}

	for (i=0; i<len; i++)
	{
		outWave[i] = (32765.0/max)*inWave[i];
	}
}

void EasyLoop(signed short* inWave, signed short* outWave, int len, double smooth)
// smooth - 0 ... 1 - relative lenght of crossfading

{
	double *in, *out;
	in = new double[len];
	out = new double[len];

	for (i=0; i<len; i++)
	{
		in[i] = inWave[i];
	}

	for (i=0; i<len; i++)
	{
		if (i < 0.5 * smooth * len)	out[i] = (1.0-(double(i)/(0.5*smooth*len)))*in[i + len/2] + (double(i)/(0.5*smooth*len))*in[i];
			else if (i > len - 0.5 * smooth * len) out[i] = (double(len-i)/(0.5*smooth*len))*in[i] + (1.0 - (double(len-i)/(0.5*smooth*len)))*in[i - len/2];
				else out [i] = in [i];
		if (out[i] < -32767) out[i] = -32767; 
		if (out[i] > 32767)  out[i] =  32767;
		outWave[i] = out[i];
	}

	delete [] in;
	delete [] out;
}

#endif